﻿using System;
using System.Collections.Generic;
using Aggregator.Aggregation.PublicHealthRepositoryApi;
using Aggregator.Aggregation.PublicHealthRepositoryApi.Clients.HealthVault;
using Aggregator.Aggregation.PublicHealthRepositoryApi.Clients.Vivaport;
using Aggregator.Persistence.Entities;

namespace Aggregator.Aggregation
{
    public static class AggregatorDictionaries
    {
        public static Dictionary<BiometricHealthDataType, Type> HealthDataTypeToType { get; set; }
        public static Dictionary<PublicHealthRepository, IPublicHealthRepositoryApiClient> PublicHealthRepositoryApis { get; set; }
        public static List<IPublicHealthRepositoryApiClient> PublicHealthRepositoryApisList { get; set; }
		public static Dictionary<DiagnosisHealthDataType, List<string>> DiagnosisTypeToIcdDictionary { get; set; }
		public static Dictionary<MedicationHealthDataType, List<string>> MedicationToEffectiveDictionary { get; set; } 

        static AggregatorDictionaries()
		{
			#region biometric types
			HealthDataTypeToType = new Dictionary<BiometricHealthDataType, Type>
            {
                {BiometricHealthDataType.BodyMassIndex, typeof (string)},
                {BiometricHealthDataType.DiastolicBloodPressure, typeof (string)},
                {BiometricHealthDataType.SystolicBloodPressure, typeof (string)}
            };
			#endregion
			#region phr api clients
			// nezinau kuri buda naudot
            PublicHealthRepositoryApis = new Dictionary<PublicHealthRepository, IPublicHealthRepositoryApiClient>
            {
                {PublicHealthRepository.Vivaport, new VivaportApiClient()},
				{PublicHealthRepository.HealthVault, new HealthVaultApiClient() }				
            };

            PublicHealthRepositoryApisList = new List<IPublicHealthRepositoryApiClient>
            {
                new VivaportApiClient(), new HealthVaultApiClient()
            };
			#endregion
			#region diagnosis to icd
			DiagnosisTypeToIcdDictionary = new Dictionary<DiagnosisHealthDataType, List<string>>
			{
				{ DiagnosisHealthDataType.AcuteKidneyInjuryDiagnosis, new List<string>
				{
					"N17.9"	
				} },
				{ DiagnosisHealthDataType.AcuteMyocardialInfarctionDiagnosis, new List<string>
				{
					
				} },
				{ DiagnosisHealthDataType.AnginaPectoris, new List<string>
				{
					"I25.10"	
				} },
				{ DiagnosisHealthDataType.AtrialFibrillationDiagnosis, new List<string>
				{
					
				} },
				{ DiagnosisHealthDataType.ChronicObstructivePulmonaryDiseaseDiagnosis, new List<string>
				{
					
				} },
				{ DiagnosisHealthDataType.DeathCardiovascular, new List<string>
				{
					
				} },
				{ DiagnosisHealthDataType.DeathDiagnosis, new List<string>
				{
					
				} },
				{ DiagnosisHealthDataType.DepressionDiagnosis, new List<string>
				{
					"Z13.89"	
				} },
				{ DiagnosisHealthDataType.DiabetesDiagnosis, new List<string>
				{
					"Z13.1"	
				} },
				{ DiagnosisHealthDataType.HeartFailureDiagnosis, new List<string>
				{
					"I97.13"	
				} },
				{ DiagnosisHealthDataType.HistoryOfCoronaryHeartDiseaseOrStroke, new List<string>
				{
					
				} },
				{ DiagnosisHealthDataType.HypertensionDiagnosis, new List<string>
				{
					
				} },
				{ DiagnosisHealthDataType.IschemicHeartDiseaseDiagnosis, new List<string>
				{
					
				} },
				{ DiagnosisHealthDataType.IschemicStrokeDiagnosis, new List<string>
				{
					
				} },
				{ DiagnosisHealthDataType.IschemicHeartDiseaseSelfHistory, new List<string>
				{
					
				} },
				{ DiagnosisHealthDataType.LeftVentricularHypertrophyDiagnosis, new List<string>
				{
					
				} },
				{ DiagnosisHealthDataType.ObstructiveSleepApneaDiagnosis, new List<string>
				{
					
				} }
			};
			#endregion

			#region	medications
			MedicationToEffectiveDictionary = new Dictionary<MedicationHealthDataType, List<string>>
			{
				{ MedicationHealthDataType.AntidepressantMedication, new List<string> { "ibuprofenum" }}
			};
			#endregion
		}
    }
}