﻿using System;
using System.Collections.Generic;
using System.Text;

namespace Aggregator.Aggregation.CarreApi
{
	public class SparqlQueryHelper
	{
		private User user;

		public static readonly Dictionary<BiometricHealthDataType, string> BiometricSuffixes;

		public static readonly Dictionary<BiometricHealthDataType, string> BiometricDatatypes;

		public static readonly Dictionary<BiometricHealthDataType, string> BiometricPredicates;

		public static readonly Dictionary<BiometricHealthDataType, string> BiometricMeasurementType;


		public static readonly Dictionary<DiagnosisHealthDataType, string> DiagnosisSuffixes;

		public static readonly Dictionary<DiagnosisHealthDataType, string> DiagnosisDatatypes;

		public static readonly Dictionary<DiagnosisHealthDataType, string> DiagnosisPredicates;

		public static readonly Dictionary<DiagnosisHealthDataType, string> DiagnosisMeasurementType;

		private const string GeneralPrefix = "PREFIX xsd: <http://www.w3.org/2001/XMLSchema#>"
											+ "PREFIX rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns#>" 
											+ "PREFIX rdfs: <http://www.w3.org/2000/01/rdf-schema#>" 
											+ "PREFIX : <http://carre.kmi.open.ac.uk/ontology/sensors.owl#>" 
											+ "PREFIX risk: <http://carre.kmi.open.ac.uk/ontology/risk.owl#>" 
											+ "PREFIX carreManufacturer: <http://carre.kmi.open.ac.uk/manufacturers/>" 
											+ "PREFIX carreUsers: <https://carre.kmi.open.ac.uk/users/> ";

		static SparqlQueryHelper()
		{
			//Kiekvienam BiometricHealthDataType reikia uzpildyti visus situs
			//Date turiu keliauti su kiekvienu is data tipu kartu

			#region biometric maps
			BiometricSuffixes = new Dictionary<BiometricHealthDataType, string>
			{
				{ BiometricHealthDataType.Date, "date"},
				{ BiometricHealthDataType.BodyMassIndex, "bodmasind"},
				{ BiometricHealthDataType.DiastolicBloodPressure, "diablopre"},
				{ BiometricHealthDataType.SystolicBloodPressure, "sysblopre"},
				{ BiometricHealthDataType.HeartRate, "heartrate"},
				{ BiometricHealthDataType.LeftVentricularMassIndex, "leftventricularmassindex"},
				{ BiometricHealthDataType.LeftVentricularWallThickness, "leftventricularwallthickness"},
				{ BiometricHealthDataType.ApnoeaHypopnoeaIndex, "apnoeahyppopnoeaindex"},
				{ BiometricHealthDataType.HighDensityLipoproteinCholesterol,	"highdensitylipoproteincholesterol"},
				{ BiometricHealthDataType.LowDensityLipoproteinCholesterol, "lowdensitylipoproteincholesterol"},
				{ BiometricHealthDataType.TotalCholesterol, "totalcholesterol"},
				{ BiometricHealthDataType.PlasmaBtypeNatriureticPeptide, "plasmabtypenatriutreticpeptride"},
				{ BiometricHealthDataType.SmokingIntensity, "smokingintensity"},
				{ BiometricHealthDataType.AnkleBranchialIndex, "anklebranchialindex"},
				{ BiometricHealthDataType.OldMyocardialInfarction, "oldmyocardialinfarction"},
				{ BiometricHealthDataType.TimeAfterAcuteMyocardialInfarctionEvent, "timeafteracutemyocardialinfarcrtionevent"},
				{ BiometricHealthDataType.HemoglobinLevel, "hemoglobinlevel"},
				{ BiometricHealthDataType.TriglyceridesSerumConcentration, "triglyceridesserumconcentration"},
				{ BiometricHealthDataType.UricAcidSerumConcentration, "uricacidserumconcentration"},
				{ BiometricHealthDataType.SerumCreatinineLevel, "serumcreatininelevel"},
				{ BiometricHealthDataType.SmokingStatus, "smokingstatus"},
				{ BiometricHealthDataType.BloodGlucoseFasting, "bloodglucosefasting"},
				{ BiometricHealthDataType.Troponin, "troponin"},
				{ BiometricHealthDataType.CreatineKinaseMbFraction, "creatinekinasembfraction"},
				{ BiometricHealthDataType.Ecg, "ecg"},
				{ BiometricHealthDataType.EjectionFraction, "ejectionfraction"}
			};

			BiometricDatatypes = new Dictionary<BiometricHealthDataType, string>
			{
				{ BiometricHealthDataType.Date, "datetime"},
				{ BiometricHealthDataType.BodyMassIndex, "float"},
				{ BiometricHealthDataType.DiastolicBloodPressure, "integer"},
				{ BiometricHealthDataType.SystolicBloodPressure, "integer"},
				{ BiometricHealthDataType.HeartRate, "string"},
				{ BiometricHealthDataType.LeftVentricularMassIndex, "string"},
				{ BiometricHealthDataType.LeftVentricularWallThickness, ""},
				{ BiometricHealthDataType.ApnoeaHypopnoeaIndex, "string"},
				{ BiometricHealthDataType.HighDensityLipoproteinCholesterol,	"string"},
				{ BiometricHealthDataType.LowDensityLipoproteinCholesterol, "string"},
				{ BiometricHealthDataType.TotalCholesterol, "string"},
				{ BiometricHealthDataType.PlasmaBtypeNatriureticPeptide, "string"},
				{ BiometricHealthDataType.SmokingIntensity, "string"},
				{ BiometricHealthDataType.AnkleBranchialIndex, "string"},
				{ BiometricHealthDataType.OldMyocardialInfarction, "string"},
				{ BiometricHealthDataType.TimeAfterAcuteMyocardialInfarctionEvent, "string"},
				{ BiometricHealthDataType.HemoglobinLevel, "string"},
				{ BiometricHealthDataType.TriglyceridesSerumConcentration, "string"},
				{ BiometricHealthDataType.UricAcidSerumConcentration, "string"},
				{ BiometricHealthDataType.SerumCreatinineLevel, "string"},
				{ BiometricHealthDataType.SmokingStatus, "string"},
				{ BiometricHealthDataType.BloodGlucoseFasting, "string"},
				{ BiometricHealthDataType.Troponin, "string"},
				{ BiometricHealthDataType.CreatineKinaseMbFraction, "string"},
				{ BiometricHealthDataType.Ecg, "string"},
				{ BiometricHealthDataType.EjectionFraction, "string"}
			};

			BiometricPredicates = new Dictionary<BiometricHealthDataType, string>
			{
				{ BiometricHealthDataType.Date, "has_date"},
				{ BiometricHealthDataType.BodyMassIndex, "has_bmi"},
				{ BiometricHealthDataType.DiastolicBloodPressure, "has_blood_pressure_diastolic"},
				{ BiometricHealthDataType.SystolicBloodPressure, "has_blood_pressure_systolic"},
				{ BiometricHealthDataType.HeartRate, "has_heart_rate"},
				{ BiometricHealthDataType.LeftVentricularMassIndex, "has_left_ventricular_mass_index"},
				{ BiometricHealthDataType.LeftVentricularWallThickness, "has_left_ventricular_wall_thickness"},
				{ BiometricHealthDataType.ApnoeaHypopnoeaIndex, "has_apnoea_–_hypopnoea_inde_(ai)"},
				{ BiometricHealthDataType.HighDensityLipoproteinCholesterol,	"has_high-density_lipoprotein_cholesterol_hdl-c"},
				{ BiometricHealthDataType.LowDensityLipoproteinCholesterol, "has_ldl-c_low-density_lipoprotein_cholesterol"},
				{ BiometricHealthDataType.TotalCholesterol, "has_total_cholesterol_tc"},
				{ BiometricHealthDataType.PlasmaBtypeNatriureticPeptide, "has_plasma_b-type_natriuretic_peptide_bnp"},
				{ BiometricHealthDataType.SmokingIntensity, "has_smoking_intensity"},
				{ BiometricHealthDataType.AnkleBranchialIndex, "has_ankle-branchial_index"},
				{ BiometricHealthDataType.OldMyocardialInfarction, "has_old_myocardial_infarction"},
				{ BiometricHealthDataType.TimeAfterAcuteMyocardialInfarctionEvent, "has_time_after_acute_myocardial_infarction_event"},
				{ BiometricHealthDataType.HemoglobinLevel, "has_hemoglobin_hb_level"},
				{ BiometricHealthDataType.TriglyceridesSerumConcentration, "has_triglycerides_serum_concentration_tg"},
				{ BiometricHealthDataType.UricAcidSerumConcentration, "has_uric_acid_serum_concentration"},
				{ BiometricHealthDataType.SerumCreatinineLevel, "has_serum_creatinine_level"},
				{ BiometricHealthDataType.SmokingStatus, "has_smoking_status"},
				{ BiometricHealthDataType.BloodGlucoseFasting, "has_blood_glucose_fasting"},
				{ BiometricHealthDataType.Troponin, "has_troponin"},
				{ BiometricHealthDataType.CreatineKinaseMbFraction, "has_creatine_kinase_mb_fraction_ck-mb"},
				{ BiometricHealthDataType.Ecg, "has_ecg"},
				{ BiometricHealthDataType.EjectionFraction, "has_ejection_fraction_ef"}
			};

			BiometricMeasurementType = new Dictionary<BiometricHealthDataType, string>
			{
				//Date neturi measurement type
				{ BiometricHealthDataType.BodyMassIndex, "individual_weight_measurement"},
				{ BiometricHealthDataType.DiastolicBloodPressure, "individual_blood_pressure_measurement"},
				{ BiometricHealthDataType.SystolicBloodPressure, "individual_blood_pressure_measurement"},
				{ BiometricHealthDataType.HeartRate, "heart_rate_measurement_type"},
				{ BiometricHealthDataType.LeftVentricularMassIndex, "left_ventricular_mass_index_measurement_type"},
				{ BiometricHealthDataType.LeftVentricularWallThickness, "left_ventricular_wall_thickness_measurement_type"},
				{ BiometricHealthDataType.ApnoeaHypopnoeaIndex, "apnoea_–_hypopnoea_inde_(ai)_measurement_type"},
				{ BiometricHealthDataType.HighDensityLipoproteinCholesterol,	"high-density_lipoprotein_cholesterol_hdl-c_measurement_type"},
				{ BiometricHealthDataType.LowDensityLipoproteinCholesterol, "ldl-c_low-density_lipoprotein_cholesterol_measurement_type"},
				{ BiometricHealthDataType.TotalCholesterol, "total_cholesterol_tc_measurement_type"},
				{ BiometricHealthDataType.PlasmaBtypeNatriureticPeptide, "plasma_b-type_natriuretic_peptide_bnp_measurement_type"},
				{ BiometricHealthDataType.SmokingIntensity, "smoking_intensity_measurement_type"},
				{ BiometricHealthDataType.AnkleBranchialIndex, "ankle-branchial_index_measurement_type"},
				{ BiometricHealthDataType.OldMyocardialInfarction, "old_myocardial_infarction_measurement_type"},
				{ BiometricHealthDataType.TimeAfterAcuteMyocardialInfarctionEvent, ""},
				{ BiometricHealthDataType.HemoglobinLevel, "time_after_acute_myocardial_infarction_event_measurement_type"},
				{ BiometricHealthDataType.TriglyceridesSerumConcentration, "triglycerides_serum_concentration_tg_measurement_type"},
				{ BiometricHealthDataType.UricAcidSerumConcentration, "uric_acid_serum_concentration_measurement_type"},
				{ BiometricHealthDataType.SerumCreatinineLevel, "serum_creatinine_level_measurement_type"},
				{ BiometricHealthDataType.SmokingStatus, "smoking_status_measurement_type"},
				{ BiometricHealthDataType.BloodGlucoseFasting, "blood_glucose_fasting_measurement_type"},
				{ BiometricHealthDataType.Troponin, "troponin_measurement_type"},
				{ BiometricHealthDataType.CreatineKinaseMbFraction, "creatine_kinase_mb_fraction_ck-mb_measurement_type"},
				{ BiometricHealthDataType.Ecg, "ecg_measurement_type"},
				{ BiometricHealthDataType.EjectionFraction, "ejection_fraction_ef_measurement_type"}
			};
			#endregion


			#region diagnosis maps
			DiagnosisSuffixes = new Dictionary<DiagnosisHealthDataType, string> {
				{ DiagnosisHealthDataType.AcuteKidneyInjuryDiagnosis, "acutekidneyinjury" },
				{ DiagnosisHealthDataType.AcuteMyocardialInfarctionDiagnosis, "acutemyocardialinfarction" },
				{ DiagnosisHealthDataType.AnginaPectoris, "anginapectoris" },
				{ DiagnosisHealthDataType.AtrialFibrillationDiagnosis, "atrialfibrillation" },
				{ DiagnosisHealthDataType.ChronicObstructivePulmonaryDiseaseDiagnosis, "chronicobstructivepulmonarydisease" },
				{ DiagnosisHealthDataType.DeathCardiovascular, "deathcardiovascular" },
				{ DiagnosisHealthDataType.DeathDiagnosis, "death" },
				{ DiagnosisHealthDataType.DepressionDiagnosis, "depression" },
				{ DiagnosisHealthDataType.DiabetesDiagnosis, "diabetes" },
				{ DiagnosisHealthDataType.HeartFailureDiagnosis, "heartfailure" },
				{ DiagnosisHealthDataType.HistoryOfCoronaryHeartDiseaseOrStroke, "historyofcoronaryheartdiesease" },
				{ DiagnosisHealthDataType.HypertensionDiagnosis, "hypertension" },
				{ DiagnosisHealthDataType.IschemicHeartDiseaseDiagnosis, "ischemicheartdisease" },
				{ DiagnosisHealthDataType.IschemicStrokeDiagnosis, "ischemicstroke" },
				{ DiagnosisHealthDataType.IschemicHeartDiseaseSelfHistory, "ischemichearthdiseaseselfhistory" },
				{ DiagnosisHealthDataType.LeftVentricularHypertrophyDiagnosis, "leftventricularhypertrophy" },
				{ DiagnosisHealthDataType.ObstructiveSleepApneaDiagnosis, "obstructivesleepapnea" }
			};

			DiagnosisDatatypes = new Dictionary<DiagnosisHealthDataType, string> {
				{ DiagnosisHealthDataType.AcuteKidneyInjuryDiagnosis, "string" },
				{ DiagnosisHealthDataType.AcuteMyocardialInfarctionDiagnosis, "string" },
				{ DiagnosisHealthDataType.AnginaPectoris, "string" },
				{ DiagnosisHealthDataType.AtrialFibrillationDiagnosis, "string" },
				{ DiagnosisHealthDataType.ChronicObstructivePulmonaryDiseaseDiagnosis, "string" },
				{ DiagnosisHealthDataType.DeathCardiovascular, "string" },
				{ DiagnosisHealthDataType.DeathDiagnosis, "string" },
				{ DiagnosisHealthDataType.DepressionDiagnosis, "string" },
				{ DiagnosisHealthDataType.DiabetesDiagnosis, "string" },
				{ DiagnosisHealthDataType.HeartFailureDiagnosis, "string" },
				{ DiagnosisHealthDataType.HistoryOfCoronaryHeartDiseaseOrStroke, "string" },
				{ DiagnosisHealthDataType.HypertensionDiagnosis, "string" },
				{ DiagnosisHealthDataType.IschemicHeartDiseaseDiagnosis, "string" },
				{ DiagnosisHealthDataType.IschemicStrokeDiagnosis, "string" },
				{ DiagnosisHealthDataType.IschemicHeartDiseaseSelfHistory, "string" },
				{ DiagnosisHealthDataType.LeftVentricularHypertrophyDiagnosis, "string" },
				{ DiagnosisHealthDataType.ObstructiveSleepApneaDiagnosis, "string" }
			};

			DiagnosisPredicates = new Dictionary<DiagnosisHealthDataType, string> {
				{ DiagnosisHealthDataType.AcuteKidneyInjuryDiagnosis, "has_acute_kidney_injury_diagnosis" },
				{ DiagnosisHealthDataType.AcuteMyocardialInfarctionDiagnosis, "has_acute_myocardial_infarction_diagnosis" },
				{ DiagnosisHealthDataType.AnginaPectoris, "has_angina_pectoris" },
				{ DiagnosisHealthDataType.AtrialFibrillationDiagnosis, "has_atrial_fibrillation_diagnosis" },
				{ DiagnosisHealthDataType.ChronicObstructivePulmonaryDiseaseDiagnosis, "has_chronic_obstructive_pulmonary_disease_diagnosis" },
				{ DiagnosisHealthDataType.DeathCardiovascular, "has_death_cardiovascular" },
				{ DiagnosisHealthDataType.DeathDiagnosis, "has_death_diagnosis" },
				{ DiagnosisHealthDataType.DepressionDiagnosis, "has_depression_diagnosis" },
				{ DiagnosisHealthDataType.DiabetesDiagnosis, "has_diabetes_diagnosis" },
				{ DiagnosisHealthDataType.HeartFailureDiagnosis, "has_heart_failure_diagnosis" },
				{ DiagnosisHealthDataType.HistoryOfCoronaryHeartDiseaseOrStroke, "has_history_of_coronary_heart_disease_chd_or_stroke" },
				{ DiagnosisHealthDataType.HypertensionDiagnosis, "has_hypertension_diagnosis" },
				{ DiagnosisHealthDataType.IschemicHeartDiseaseDiagnosis, "has_ischemic_heart_disease_diagnosis" },
				{ DiagnosisHealthDataType.IschemicStrokeDiagnosis, "has_ischemic_stroke_diagnosis" },
				{ DiagnosisHealthDataType.IschemicHeartDiseaseSelfHistory, "has_ischemic_heart_disease_self_history" },
				{ DiagnosisHealthDataType.LeftVentricularHypertrophyDiagnosis, "has_left_ventricular_hypertrophy_diagnosis" },
				{ DiagnosisHealthDataType.ObstructiveSleepApneaDiagnosis, "has_obstructive_sleep_apnea_diagnosis" }
			};

			DiagnosisMeasurementType = new Dictionary<DiagnosisHealthDataType, string> {
				{ DiagnosisHealthDataType.AcuteKidneyInjuryDiagnosis, "acute_kidney_injury_diagnosis_measurement_type" },
				{ DiagnosisHealthDataType.AcuteMyocardialInfarctionDiagnosis, "acute_myocardial_infarction_diagnosis_measurement_type" },
				{ DiagnosisHealthDataType.AnginaPectoris, "angina_pectoris_measurement_type" },
				{ DiagnosisHealthDataType.AtrialFibrillationDiagnosis, "atrial_fibrillation_diagnosis_measurement_type" },
				{ DiagnosisHealthDataType.ChronicObstructivePulmonaryDiseaseDiagnosis, "chronic_obstructive_pulmonary_disease_diagnosis_measurement_type" },
				{ DiagnosisHealthDataType.DeathCardiovascular, "death_cardiovascular_measurement_type" },
				{ DiagnosisHealthDataType.DeathDiagnosis, "death_diagnosis_measurement_type" },
				{ DiagnosisHealthDataType.DepressionDiagnosis, "depression_diagnosis_measurement_type" },
				{ DiagnosisHealthDataType.DiabetesDiagnosis, "diabetes_diagnosis_measurement_type" },
				{ DiagnosisHealthDataType.HeartFailureDiagnosis, "heart_failure_diagnosis_measurement_type" },
				{ DiagnosisHealthDataType.HistoryOfCoronaryHeartDiseaseOrStroke, "history_of_coronary_heart_disease_chd_or_stroke_measurement_type" },
				{ DiagnosisHealthDataType.HypertensionDiagnosis, "hypertension_diagnosis_measurement_type" },
				{ DiagnosisHealthDataType.IschemicHeartDiseaseDiagnosis, "ischemic_heart_disease_diagnosis_measurement_type" },
				{ DiagnosisHealthDataType.IschemicStrokeDiagnosis, "ischemic_stroke_diagnosis_measurement_type" },
				{ DiagnosisHealthDataType.IschemicHeartDiseaseSelfHistory, "ischemic_heart_disease_self_history_measurement_type" },
				{ DiagnosisHealthDataType.LeftVentricularHypertrophyDiagnosis, "left_ventricular_mass_index_measurement_type" },
				{ DiagnosisHealthDataType.ObstructiveSleepApneaDiagnosis, "obstructive_sleep_apnea_diagnosis_measurement_type" }
			};
			#endregion
		}

		public SparqlQueryHelper(User user)
		{
			if (user == null)
			{
				throw new ArgumentException("User cant be null");
			}

			this.user = user;
		}

		public string GetGeneralPrefix()
		{
			return GeneralPrefix + "INSERT INTO <https://carre.kmi.open.ac.uk/users/" + user.Username + ">";
		}

		public string GetUniqueMeasurement()
		{
			var timestamp = DateTime.Now.ToString("yyyyMMddHHmmssfff");
			return "https://carre.kmi.open.ac.uk./users/" + user.Username + "/measurements/phr" + timestamp;
		}

		public string PrepareInsertQuery(ApiRequest request)
		{
			if (request.Type == RequestType.Biometric)
			{
				return PrepareBiometricInsertQuery(request);
			}
			else
			{
				return PrepareDiagnosisInsertQuery(request);
			}
		}

		private string PrepareDiagnosisInsertQuery(ApiRequest request)
		{
			var uniqueMeasurement = GetUniqueMeasurement();

			var query = new StringBuilder();
			query.Append(GetGeneralPrefix());
			query.Append("{ ");
			query.Append("<" + uniqueMeasurement + "> rdfs:type :" + DiagnosisMeasurementType[request.DiagnosisHealthDataType] + " ;");
			query.Append(":is_measured_by <http://carre.kmi.open.ac.uk/manufacturers/carre_manual_phr> ;"); //constanta visiem reikia
			query.Append(":" + BiometricSuffixes[BiometricHealthDataType.Date] + " <" + uniqueMeasurement + "_" +
						 DiagnosisSuffixes[request.DiagnosisHealthDataType] + ">;");
			query.Append(":" + DiagnosisPredicates[request.DiagnosisHealthDataType] + " <" + uniqueMeasurement + "_" +
						 DiagnosisSuffixes[request.DiagnosisHealthDataType] + ">.");
			query.Append("<" + uniqueMeasurement + "_" + BiometricSuffixes[BiometricHealthDataType.Date] + "> :has_value \"" +
						 request.Date.ToString("yyyy-MM-ddTHH:mm:ss") + "\"^^xsd:" + BiometricSuffixes[BiometricHealthDataType.Date] + " .");
			query.Append("<" + uniqueMeasurement + "_" + DiagnosisSuffixes[request.DiagnosisHealthDataType] + "> :has_value \"" +
						 request.Value + "\"^^xsd:" + DiagnosisDatatypes[request.DiagnosisHealthDataType] + " .");
			query.Append("}");
			return query.ToString();			
		}

		private string PrepareBiometricInsertQuery(ApiRequest request)
		{
			var uniqueMeasurement = GetUniqueMeasurement();

			var query = new StringBuilder();
			query.Append(GetGeneralPrefix());
			query.Append("{ ");
			query.Append("<" + uniqueMeasurement + "> rdfs:type :" + BiometricMeasurementType[request.BiometricHealthDataType] + " ;");
			query.Append(":is_measured_by <http://carre.kmi.open.ac.uk/manufacturers/carre_manual_phr> ;"); //constanta visiem reikia
			query.Append(":" + BiometricPredicates[BiometricHealthDataType.Date] + " <" + uniqueMeasurement + "_" +
						 BiometricSuffixes[request.BiometricHealthDataType] + ">;");
			query.Append(":" + BiometricPredicates[request.BiometricHealthDataType] + " <" + uniqueMeasurement + "_" +
						 BiometricSuffixes[request.BiometricHealthDataType] + ">.");
			query.Append("<" + uniqueMeasurement + "_" + BiometricSuffixes[BiometricHealthDataType.Date] + "> :has_value \"" +
						 request.Date.ToString("yyyy-MM-ddTHH:mm:ss") + "\"^^xsd:" + BiometricDatatypes[BiometricHealthDataType.Date] + " .");
			query.Append("<" + uniqueMeasurement + "_" + BiometricSuffixes[request.BiometricHealthDataType] + "> :has_value \"" +
						 request.Value + "\"^^xsd:" + BiometricDatatypes[request.BiometricHealthDataType] + " .");
			query.Append("}");
			return query.ToString();
		}
	}
}