﻿using System;
using System.Linq;
using Aggregator.Aggregation.CarreApi;
using Aggregator.Persistence;
using Aggregator.Persistence.Entities;
using Microsoft.Health.PatientConnect;

namespace Aggregator.Aggregation.PublicHealthRepositoryApi.Clients.HealthVault
{
	public class HealthVaultPatientSync : IDisposable
	{
		private readonly AggregatorDbContext _database;
		private readonly ApiClient _api;

		public HealthVaultPatientSync()
		{
			_database = new AggregatorDbContext();			
			_api = new ApiClient();
		}

		public int SyncPatients()
		{
		    var connection = OfflineWebAppConnectionFactory.GetConnection(Guid.Empty);

			connection.Authenticate();

			var authorizedConnections = PatientConnection.GetValidatedConnections(connection);

			foreach (var authorizedConnection in authorizedConnections)
			{
				var personId = authorizedConnection.PersonId;

				var carreId = authorizedConnection.ApplicationPatientId;
				if (!
					_database.RegisteredPublicHealthRepositoryTokens.Any(
						t => t.PublicHealthRepository == PublicHealthRepository.HealthVault && t.CarreToken == carreId))
				{
					_database.RegisteredPublicHealthRepositoryTokens.Add(new RegisteredPublicHealthRepositoryToken
					{
						CarreToken = carreId,
						PublicHealthRepository = PublicHealthRepository.HealthVault,
						PublicHealthRepositoryToken = personId.ToString(),
						RegistrationTime = DateTime.Now
					});

					_api.SendConnectionCallback(new ConnectionCallback
					{
						PublicHealthRepository = "healthvault",
						CarreToken = carreId,
						Email = "",
						PublicHealthRepositoryToken = personId.ToString(),
						Username = ""
					});
				}
			}

			return _database.SaveChanges();
		}

		public void Dispose()
		{
			_database.Dispose();
		}
	}
}