﻿using System;
using System.Collections.Generic;
using System.Configuration;
using System.Linq;
using System.Net.Http;
using System.Text;
using System.Web.Script.Serialization;
using Aggregator.Aggregation.PublicHealthRepositoryApi.Clients.Vivaport.HealthDataResponses;

namespace Aggregator.Aggregation.PublicHealthRepositoryApi.Clients.Vivaport
{
	public class VivaportApiClient : IPublicHealthRepositoryApiClient
	{
		public static readonly Dictionary<BiometricHealthDataType, string> UrlBiometricHealthDataTypeDictionary;
		public static readonly Dictionary<DiagnosisHealthDataType, string> UrlDiagnosisHealthDataTypeDictionary;
		public static readonly Dictionary<MedicationHealthDataType, string> UrlMedicationHealthDataTypeDictionary;

		static VivaportApiClient()
		{
			UrlBiometricHealthDataTypeDictionary = new Dictionary<BiometricHealthDataType, string>
			{
				{ BiometricHealthDataType.BodyMassIndex, "BodyMassIndexRequest" },
				{ BiometricHealthDataType.DiastolicBloodPressure, "DiastolicBloodPressureRequest" },
				{ BiometricHealthDataType.SystolicBloodPressure, "SystolicBloodPressureRequest" }
			};			

			UrlDiagnosisHealthDataTypeDictionary = new Dictionary<DiagnosisHealthDataType, string>
			{
				{ DiagnosisHealthDataType.AcuteKidneyInjuryDiagnosis, "AcuteKidneyInjury" }
			};

			UrlMedicationHealthDataTypeDictionary = new Dictionary<MedicationHealthDataType, string>
			{
				{ MedicationHealthDataType.AntidepressantMedication, "ibuprofenum"}
			};
		}

		public bool DataMapExists(BiometricHealthDataType type)
		{
			return UrlBiometricHealthDataTypeDictionary.ContainsKey(type);
		}

		public bool DataMapExists(DiagnosisHealthDataType type)
		{
			return UrlDiagnosisHealthDataTypeDictionary.ContainsKey(type);
		}

		public bool DataMapExists(MedicationHealthDataType type)
		{
			return UrlMedicationHealthDataTypeDictionary.ContainsKey(type);
		}

		public IList<HealthDataDto> GetHealthData(BiometricHealthDataType type, string token, DateTime from)
		{
			using (var httpClient = new HttpClient()) {
				try
				{
					var content = new StringContent(VivaportJsonQueryHelper.GetJsonStringForDataRequest(token, from),
						Encoding.UTF8, "application/json");

					var result =
						httpClient.PostAsync(
							ConfigurationManager.AppSettings["VivaportApiBaseUrl"] + "json/oneway/" + UrlBiometricHealthDataTypeDictionary[type] +
							"?format=json", content)
							.Result;

					var serializer = new JavaScriptSerializer();
					var responseString = result.Content.ReadAsStringAsync().Result;
					var response =
						(IList<HealthDataResponse>) serializer.Deserialize(responseString, typeof (List<HealthDataResponse>));

					return response.Select(respo => new HealthDataDto(respo.HealthDataValue, respo.Date)).ToList();
				}
				catch (Exception e)
				{
					return new List<HealthDataDto>();
				}
			}
		}


		public IList<HealthDataDto> GetDiagnosisHealthData(string token, DateTime from)
		{
			using (var httpClient = new HttpClient()) {
				try {
					var content = new StringContent(VivaportJsonQueryHelper.GetJsonStringForDataRequest(token, from),
						Encoding.UTF8, "application/json");

					var result =
						httpClient.PostAsync(
							ConfigurationManager.AppSettings["VivaportApiBaseUrl"] + "json/oneway/DiagnosisRequest?format=json", content)
							.Result;

					var serializer = new JavaScriptSerializer();
					var responseString = result.Content.ReadAsStringAsync().Result;
					var response =
						(IList<DiagnosisResponse>)serializer.Deserialize(responseString, typeof(List<DiagnosisResponse>));

					return response.Select(respo => new HealthDataDto(respo.Diagnosis, respo.Date)).ToList();
				} catch (Exception e) {
					return new List<HealthDataDto>();
				}
			}	
		}

		public IList<HealthDataDto> GetMedicationHealthData(string token, DateTime @from)
		{
			using (var httpClient = new HttpClient()) {
				try {
					var content = new StringContent(VivaportJsonQueryHelper.GetJsonStringForDataRequest(token, from),
						Encoding.UTF8, "application/json");

					var result =
						httpClient.PostAsync(
							ConfigurationManager.AppSettings["VivaportApiBaseUrl"] + "json/oneway/MedicationRequest?format=json", content)
							.Result;

					var serializer = new JavaScriptSerializer();
					var responseString = result.Content.ReadAsStringAsync().Result;
					var response =
						(IList<MedicationResponse>)serializer.Deserialize(responseString, typeof(List<MedicationResponse>));

					return response.Select(respo => new HealthDataDto(respo.Medication, respo.Date)).ToList();
				} catch (Exception e) {
					return new List<HealthDataDto>();
				}
			}
		}

	}

}