﻿using System;
using System.Linq;
using Aggregator.Aggregation.CarreApi;
using Aggregator.Persistence;
using Aggregator.Persistence.Entities;
using ServiceStack.Logging;

namespace Aggregator.Aggregation
{
	/// <summary>
	/// Klasė atliekanti pagrindinį agregatoriaus funkcionalumą
	/// </summary>
	public class PublicHealthRepositoryDataAggregator
	{
		/// <summary>
		/// Metodas skirtas surinkti duomenis iš repozitorijų ir juos perkelti i Carre
		/// </summary>
		/// 
		private ILog logger;
		public void Aggregate()
		{
			var carreClient = new ApiClient();
			var biometricHealthDataTypeValues = Enum.GetValues(typeof(BiometricHealthDataType)).Cast<BiometricHealthDataType>().ToList();
			var diagnosisHealthDataTypeValues = Enum.GetValues(typeof(DiagnosisHealthDataType)).Cast<DiagnosisHealthDataType>().ToList();
			var medicationHealthDataTypeValues = Enum.GetValues(typeof(MedicationHealthDataType)).Cast<MedicationHealthDataType>().ToList();
			var publicHealthRepositoriesApis = Enum.GetValues(typeof(PublicHealthRepository)).Cast<PublicHealthRepository>().ToList();
			var database = new AggregatorDbContext();
			logger = LogManager.GetLogger(GetType());

			foreach (var publicHealthRepository in publicHealthRepositoriesApis) {
				var publicHealthRepositoryApiClient = AggregatorDictionaries.PublicHealthRepositoryApis[publicHealthRepository];
				var repository = publicHealthRepository;
				var tokenInfoList = database.RegisteredPublicHealthRepositoryTokens.Where(t => t.PublicHealthRepository == repository);

				foreach (var tokenInfo in tokenInfoList) {
					var fromDate =
								database.AggregatedDataRecords.OrderByDescending(a => a.Date).FirstOrDefault() == null
									? DateTime.MinValue
									: database.AggregatedDataRecords.OrderByDescending(a => a.Date).FirstOrDefault().Date;

					// Biometriniu duomenu agregavimas
					foreach (var biometricHealthDataType in biometricHealthDataTypeValues) {
						if (!publicHealthRepositoryApiClient.DataMapExists(biometricHealthDataType)
							|| !carreClient.HasDataTypeMapped(biometricHealthDataType)) {
							continue;
						}

						var healthData = 
							publicHealthRepositoryApiClient.GetHealthData(biometricHealthDataType, tokenInfo.PublicHealthRepositoryToken,
								fromDate);

						logger.Info(publicHealthRepository + " RECIEVED: " + biometricHealthDataType + " COUNT: " + healthData.Count);

						foreach (var data in healthData) {
							carreClient.UploadData(biometricHealthDataType,
									data.Value, tokenInfo.CarreToken, data.Date);
						}
					}


					// Diagnoziniu duomenu agregavimas

					var diagnosisHealthData =
							publicHealthRepositoryApiClient.GetDiagnosisHealthData(tokenInfo.PublicHealthRepositoryToken,
								fromDate);

					foreach (var diagnosisHealthDataType in diagnosisHealthDataTypeValues)
					{
						if (!publicHealthRepositoryApiClient.DataMapExists(diagnosisHealthDataType)
							|| !carreClient.HasDataTypeMapped(diagnosisHealthDataType)) {
							continue;
						}

						var type = diagnosisHealthDataType;
						var selectedDiagnosisData = diagnosisHealthData.Where(
							d => AggregatorDictionaries.DiagnosisTypeToIcdDictionary[type].Contains(d.Value));

						logger.Info("TOKEN:" + tokenInfo.PublicHealthRepositoryToken + " CARRE:" + publicHealthRepository + " RECIEVED: " + selectedDiagnosisData.Count());

						foreach (var data in selectedDiagnosisData) {
							carreClient.UploadData(diagnosisHealthDataType,
									data.Value, tokenInfo.CarreToken, data.Date);
						}	
					}

					// Medikamentu agregavimas

					var medicationHealthdata =
							publicHealthRepositoryApiClient.GetMedicationHealthData(tokenInfo.PublicHealthRepositoryToken,
								fromDate);

					//foreach (var medicationHealthDataType in medicationHealthDataTypeValues) {
					//	if (!publicHealthRepositoryApiClient.DataMapExists(medicationHealthDataType)
					//		|| !carreClient.HasDataTypeMapped(medicationHealthDataType)) {
					//		continue;
					//	}

					//	var type = medicationHealthDataType;
					//	var selectedDiagnosisData = medicationHealthdata.Where(
					//		d => AggregatorDictionaries.MedicationToEffectiveDictionary[type].Contains(d.Value));

					//	logger.Info("TOKEN:" + tokenInfo.PublicHealthRepositoryToken + " CARRE:" + publicHealthRepository + " RECIEVED: " + selectedDiagnosisData.Count());

					//	foreach (var data in selectedDiagnosisData) {
					//		carreClient.UploadData(medicationHealthDataType,
					//				data.Value, tokenInfo.CarreToken, data.Date);
					//	}
					//}


					database.AggregatedDataRecords.Add(new AggregatedDataRecord {
						CarreToken = tokenInfo.CarreToken,
						PublicHealthRepository = publicHealthRepository,
						Date = DateTime.Now
					});
				}
				database.SaveChanges();
			}
		}
	}
}