﻿using System;
using System.Linq;
using Aggregator.Aggregation.PublicHealthRepositoryApi.Clients.HealthVault;
using Aggregator.Persistence;
using Aggregator.Persistence.Entities;
using Microsoft.Health.PatientConnect;
using ServiceStack;

namespace Aggregator.Services.PhrRegistration
{
	public class HealthVaultConnectService : Service
	{
		private readonly AggregatorDbContext _database;

		public HealthVaultConnectService()
		{
			_database = new AggregatorDbContext();			
		}

		public HealthVaultConnectResponse Post(HealthVaultConnectRequest request)
		{
			var healthVaultPatientConnection =
				_database.HealthVaultPatientConnections.FirstOrDefault(c => c.CarreToken == request.CarreToken);

			if (healthVaultPatientConnection != null)
			{
				return new HealthVaultConnectResponse
				{
					SecretQuestion = healthVaultPatientConnection.Question,
					SecretAnswer = healthVaultPatientConnection.Answer,
					ValidationToken = healthVaultPatientConnection.Token
				};
			}

		    var connection = OfflineWebAppConnectionFactory.GetConnection(Guid.Empty);

			connection.Authenticate();
		
			var token = PatientConnection.Create(
				connection, "Carre", request.SecretQuestion, request.SecretAnswer, null, request.CarreToken);

			_database.HealthVaultPatientConnections.Add(new HealthVaultPatientConnection()
			{
				Answer = request.SecretAnswer,
				CarreToken = request.CarreToken,
				Date = DateTime.Now,
				Question = request.SecretQuestion,
				Token = token
			});

			_database.SaveChanges();

			return new HealthVaultConnectResponse
			{
				SecretQuestion = request.SecretQuestion,
				SecretAnswer = request.SecretAnswer,
				ValidationToken = token
			};
		}

		public override void Dispose()
		{
			_database.Dispose();
			base.Dispose();	
		}
	}
}