﻿using System;
using System.Linq;
using System.Net;
using Aggregator.Aggregation.CarreApi;
using Aggregator.Persistence;
using Aggregator.Persistence.Entities;
using ServiceStack;

namespace Aggregator.Services.PhrRegistration
{
	public class PhrRegistrationCompletionService : Service
	{
		
		public object Post(PhrRegistrationCompletionRequest completionRequest)
		{
			if (completionRequest.CarreToken.IsNullOrEmpty() || completionRequest.PublicHealthRepositoryToken.IsNullOrEmpty())
			{
				throw new ArgumentException("Invalid request");
			}

			using (var db = new AggregatorDbContext())
			{
				var entry = db.RegisteredPublicHealthRepositoryTokens.FirstOrDefault(e => e.CarreToken == completionRequest.CarreToken);

				if (entry == null)
				{
					db.RegisteredPublicHealthRepositoryTokens.Add(new RegisteredPublicHealthRepositoryToken
					{
						CarreToken = completionRequest.CarreToken,
						RegistrationTime = DateTime.Now,
						PublicHealthRepository = completionRequest.PublicHealthRepository,
						PublicHealthRepositoryToken = completionRequest.PublicHealthRepositoryToken
					});
				}
				else
				{
					entry.PublicHealthRepositoryToken = completionRequest.PublicHealthRepositoryToken;
				}

				db.SaveChanges();
			}


			using (var client = new ApiClient())
			{
				var callback = new ConnectionCallback
				{
					CarreToken = completionRequest.CarreToken,
					PublicHealthRepository = "healthvault",
					Email = "n/a",
					PublicHealthRepositoryToken = completionRequest.PublicHealthRepositoryToken,
					Username = "n/a"
				};

				if (completionRequest.PublicHealthRepository == PublicHealthRepository.HealthVault)
				{
					callback.PublicHealthRepository = "healthvault";
				}
				else if (completionRequest.PublicHealthRepository == PublicHealthRepository.Vivaport)
				{
					callback.PublicHealthRepository = "phr";
				}
				else
				{					 
					//something bad
				}

				client.SendConnectionCallback(callback);
			}

			return new HttpResult
			{
				StatusCode = HttpStatusCode.NoContent
			};
		}

	}
}