﻿using System;
using System.Collections.Generic;
using System.Configuration;
using System.Net;
using Aggregator.Persistence.Entities;
using ServiceStack;

namespace Aggregator.Services.PhrRegistration
{
	public class PhrRegistrationInitiationService : Service
	{
		private Dictionary<PublicHealthRepository, string> PublicHealthRepositoriesAuthenticationUrlMap { get; set; }

		public PhrRegistrationInitiationService()
		{
			PublicHealthRepositoriesAuthenticationUrlMap = new Dictionary<PublicHealthRepository, string>
			{
				{PublicHealthRepository.Vivaport, ConfigurationManager.AppSettings["VivaportAuthenticateUrl"]},
				{PublicHealthRepository.HealthVault, ConfigurationManager.AppSettings["HealthVaultAuthenticateUrl"]}
			};
		}

		public object Any(PhrRegistrationInitiationRequest initiationRequest)
		{
			if (!PublicHealthRepositoriesAuthenticationUrlMap.ContainsKey(initiationRequest.PublicHealthRepository)) {
				throw new ArgumentException("Public health respository is not listed within the system.");
			}

			if (initiationRequest.CarreToken.IsNullOrEmpty()) {
				throw new ArgumentException("Invalid token");
			}

			if (initiationRequest.PublicHealthRepository == PublicHealthRepository.Vivaport) {
				return new HttpResult {
					StatusCode = HttpStatusCode.Redirect,
					Headers =
					{
						{
							HttpHeaders.Location, PublicHealthRepositoriesAuthenticationUrlMap[initiationRequest.PublicHealthRepository]
							                      + "?remoteToken=" + initiationRequest.CarreToken
						}
					}
				};
			} else if (initiationRequest.PublicHealthRepository == PublicHealthRepository.HealthVault)
			{
				return new HttpResult {
					StatusCode = HttpStatusCode.Redirect,
					Headers =
					{
						{
							HttpHeaders.Location, ConfigurationManager.AppSettings["HealthVaultConnectAppUrl"] + "Home/Connect?carreToken="
								+ initiationRequest.CarreToken
						}
					}
				};
			} else {
				throw new ArgumentException("PHR not supported");
			}
		}
	}

}